"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ErrorMapperBuilder = void 0;
const builder_1 = require("@pallad/builder");
const app_env_1 = require("@pallad/app-env");
const boom = require("@hapi/boom");
class ErrorMapperBuilder extends builder_1.Builder {
    constructor(options) {
        super();
        this.options = options;
        this.errorMappers = [];
        this.onUnknownErrorListeners = [];
        this.outputTransformers = [];
        this.runIf(this.options.showStackTrace, () => {
            this.registerOutputTransformer((output, error) => {
                if ('stack' in error) {
                    return {
                        ...output,
                        payload: {
                            ...output.payload,
                            stack: error.stack
                        }
                    };
                }
                return output;
            });
        });
        this.registerOutputTransformer((output, error) => {
            var _a;
            const code = boom.isBoom(error) ? (_a = error.data) === null || _a === void 0 ? void 0 : _a.code : error.code;
            if (code) {
                return {
                    ...output,
                    payload: {
                        ...output.payload,
                        code
                    }
                };
            }
            return output;
        });
        this.runIf(!this.options.showUnknownErrorMessage, () => {
            this.registerOutputTransformer((output, error, boomedError) => {
                if (boomedError && boomedError.output.statusCode === 500) {
                    return {
                        ...output,
                        payload: {
                            ...output.payload,
                            message: 'Internal server error. Please try again later.'
                        }
                    };
                }
                return output;
            });
        });
    }
    static fromEnv() {
        return new ErrorMapperBuilder({
            showStackTrace: app_env_1.isDevelopment || app_env_1.isTest,
            showUnknownErrorMessage: app_env_1.isDevelopment || app_env_1.isTest
        });
    }
    onUnknownError(listener) {
        this.onUnknownErrorListeners.push(listener);
        return this;
    }
    registerErrorMapper(mapper) {
        this.errorMappers.push(mapper);
        return this;
    }
    registerOutputTransformer(outputTransformer) {
        this.outputTransformers.push(outputTransformer);
        return this;
    }
    get() {
        const mapToBoom = (err) => {
            if (boom.isBoom(err)) {
                return err;
            }
            for (const errorMapper of this.errorMappers) {
                const mappingResult = errorMapper(err);
                if (mappingResult) {
                    return mappingResult;
                }
            }
            for (const onUnknownErrorListener of this.onUnknownErrorListeners) {
                onUnknownErrorListener(err);
            }
            const internal = boom.internal(err.message);
            internal.reformat(true);
            return internal;
        };
        return (err) => {
            const boomedError = mapToBoom(err);
            return this.outputTransformers.reduce((output, transformer) => {
                return transformer(output, err, boomedError);
            }, boomedError.output);
        };
    }
}
exports.ErrorMapperBuilder = ErrorMapperBuilder;
